/**
 * Controller to manipulate create layout options widget
 *
 * @class
 * @since 3.0.0
 *
 * @param {MessageService}                             MessageService                             Service to show messages
 * @param {ZhLayoutPreferences}                        ZhLayoutPreferences                        Service to handle layout preferences methods
 * @param {ScreenService}                              ScreenService                              Service to manipulate screen
 * @param {ZhLayoutWidgetsBuilder}                     ZhLayoutWidgetsBuilder                     Service to create widgets to handle layouts
 * @param {ZHPromise}                                  ZHPromise              Service to handle promises
 */
var ZhLayoutCreateLayoutOptionsController = function(MessageService, ZhLayoutPreferences, ScreenService, ZhLayoutWidgetsBuilder, metaDataFactory, ZHPromise){

    var fieldDefaultProperties = {
        isVisible: true,
        showOnList: true,
        visibility: true,
        grouped: false,
        fixed: false
    };

    var createLayoutOptionsWidgetPromise = null;

    function removeConflictingProperties(field){
        var conflictingProperties = ['selectWidget', 'inlineEditable'];
        conflictingProperties.forEach(function(property){
            delete field[property];
        });
    }

    function factoryFields(parentWidget, gridFields){
        return gridFields.filter(function(field){
            return !field.skipLayout;
        }).map(function(field){
            var factoredField = metaDataFactory.fieldFactory(field, parentWidget);
            removeConflictingProperties(factoredField);
            return factoredField;
        });
    }

    /**
     * Retrieve widget that handles create layout options
     *
     * @since 3.0.0
     *
     * @param  {Layout} preference   Layout to be changed
     * @param  {Widget} configWidget Widget that handles layouts configuration
     *
     * @return {Widget}              Widget that handles create layout options
     */
    this.getCreateLayoutOptionsWidget = function(preference, configWidget){
        if(createLayoutOptionsWidgetPromise === null){
            createLayoutOptionsWidgetPromise = ZhLayoutWidgetsBuilder.factoryBuildLayoutOptionsWidget();
        }

        return createLayoutOptionsWidgetPromise.then(function(layoutOptionsWidget){
            var label = preference.LABEL_TO_PREFERENCE;

            var listWidget = configWidget.listWidget;

            layoutOptionsWidget.preference = preference;
            layoutOptionsWidget.label = ScreenService.i18n('Layout');
            if(label){
                layoutOptionsWidget.label = layoutOptionsWidget.label + ' - ' + label;
            }
            layoutOptionsWidget.layoutName = label;
            layoutOptionsWidget.reportTitle = listWidget.label;
            layoutOptionsWidget.reportOrientation = preference.PDF_REPORT_ORIENTATION;

            // get only the first two rows from listWidget
            layoutOptionsWidget.dataSource.data = Util.clone(listWidget.dataSource.data.slice(0,2));

            // get fields from listWidget
            layoutOptionsWidget.fields = factoryFields(layoutOptionsWidget, listWidget.fields);

            var layoutChanges = JSON.parse(preference.LAYOUT_CHANGES);
            var widgetDifferences = getEditingWidgetPath(layoutChanges.widgets, listWidget);

            layoutOptionsWidget.fields = layoutOptionsWidget.fields.map(function(field, index){
                delete field.__isSelected;
                delete field.__leftActions;
                delete field.__notSelectable;
                delete field.__rightActions;

                if(!Util.isEmptyOrBlank(widgetDifferences.fields)){
                    var fieldPreference = filterObject(widgetDifferences.fields, field.id, "id");
                    if(fieldPreference.obj && Object.keys(fieldPreference.obj).length){
                        return Util.extend(field, fieldPreference.obj);
                    }
                }

                return Util.extend(fieldDefaultProperties, field, true);
            });

            return layoutOptionsWidget;
        });
    };

    /**
     * Save and apply layout
     *
     * @since 3.0.0
     *
     * @param  {Widget} createLayoutOptionsWidget Widget that handles layout options
     *
     * @return {Promise}                          Promise resolved when the layout is applied
     */
    this.saveAndApplyLayout = function(createLayoutOptionsWidget){
        if(createLayoutOptionsWidget.gridConfigWidget.isValid()){
            var configWidget = createLayoutOptionsWidget.createNewLayoutWidget.configWidget;
            var listWidget = configWidget.listWidget;

            var fields = getFieldEditingWidgetAsDataSource(createLayoutOptionsWidget);
            var layout = createLayoutOptionsWidget.preference;

            var layoutChanges = JSON.parse(layout.LAYOUT_CHANGES);
            var editingWidget = getEditingWidgetPath(layoutChanges.widgets, listWidget);
            editingWidget.fields = fields;

            layout.PDF_REPORT_ORIENTATION = createLayoutOptionsWidget.reportOrientation;
            layout.LABEL_TO_PREFERENCE = createLayoutOptionsWidget.layoutName;
            layout.LAYOUT_CHANGES = JSON.stringify(layoutChanges);

            return ZhLayoutPreferences.saveLayouts([layout]).then(function(layouts){
                return ZhLayoutPreferences.applyLayout(listWidget, layouts[0]);
            }).then(function(){
                ScreenService.closePopup(true);
                ZhLayoutPreferences.updateLayouts(listWidget.id, configWidget.dataSource);
                listWidget.activate();
            }).catch(function(error){
                MessageService.showMessageByCode('zh-layout-preferences#save_error', {
                    'ERROR': error.message
                });
            });
        } else {
            return ZHPromise.reject("Invalid params");
        }
    };

    /**
     * Change field visibility
     *
     * @since 3.0.0
     *
     * @param {Field} field Field to change the visibility
     */
    this.updateFieldVisibility = function(field){
        field.isVisible = true;
        field.showOnList = field.isVisibleOnReport = field.visibility;
    };

    function getFieldEditingWidgetAsDataSource(listWidget){
        return listWidget.fields.map(function(field){
            var data = {
                'id': field.id,
                'name': field.name,
                'label': field.label,
                'labelOnReport': field.labelOnReport || null,
                'expression': field.expression || null,
                'align': field.align || null,
                'isVisible': field.isVisible,
                'showOnList': field.showOnList,
                'minWidth': field.gridColumnSize || null,
                'maxWidth': field.gridColumnSize || null,
                'gridColumnSize': field.gridColumnSize || null,
                'reportColumnSize': field.reportColumnSize || null,
                'isVisibleOnReport': field.isVisibleOnReport,
                'sequenceInList': field.sequenceInList,
                'visibility': field.isVisible && field.showOnList,
                'grouped': field.grouped || false,
                'groupOpened': field.groupOpened,
                'fixed': field.fixed || false,
                'skipLayout': field.skipLayout || false
            };
            return data;
        }).filter(function(field){
            return !field.skipLayout;
        });
    }

    function getEditingWidgetPath(widgetsArray, listWidget){
        var foundWidget = getWidgetObjectByID(widgetsArray, listWidget.id);
        if(!foundWidget){
            widgetsArray.some(function(eachWidget){
                foundWidget = getEditingWidgetPath(eachWidget.widgets, listWidget);
                return foundWidget;
            });
        }
        return foundWidget;
    }

    function filterObject(obj, value, key){
        var filtered = obj.filter(function(preference){
            return preference[key] == value;
        });

        var toReturn = {};
        toReturn.index = obj.indexOf(filtered[0]);
        toReturn.obj = filtered[0];
        return toReturn;
    }

    function getWidgetObjectByID(widgets, widgetID){
        return widgets.filter(function(eachWidgetObject){
            return eachWidgetObject.id == widgetID;
        }).shift();
    }

};

ZeedhiController.controller('ZhLayoutCreateLayoutOptionsController', ZhLayoutCreateLayoutOptionsController);

Configuration(function(ContextRegister){
    ContextRegister.register('ZhLayoutCreateLayoutOptionsController', ZhLayoutCreateLayoutOptionsController);
});